@@ -147,6 +147,22 @@ static const CidConfig cid_qmi_config [MBIM_CID_QMI_LAST] = {
     { SET, NO_QUERY, NO_NOTIFY }, /* MBIM_CID_QMI_MSG */
 };
 
+/* Note: index of the array is CID-1 */
+#define MBIM_CID_ATDS_LAST MBIM_CID_ATDS_REGISTER_STATE
+static const CidConfig cid_atds_config [MBIM_CID_ATDS_LAST] = {
+    { NO_SET, QUERY, NO_NOTIFY }, /* MBIM_CID_ATDS_SIGNAL */
+    { NO_SET, QUERY, NO_NOTIFY }, /* MBIM_CID_ATDS_LOCATION */
+    { SET,    QUERY, NO_NOTIFY }, /* MBIM_CID_ATDS_OPERATORS */
+    { SET,    QUERY, NO_NOTIFY }, /* MBIM_CID_ATDS_RAT */
+    { NO_SET, QUERY, NO_NOTIFY }, /* MBIM_CID_ATDS_REGISTER_STATE */
+};
+
+/* Note: index of the array is CID-1 */
+#define MBIM_CID_INTEL_FIRMWARE_UPDATE_LAST MBIM_CID_INTEL_FIRMWARE_UPDATE_MODEM_REBOOT
+static const CidConfig cid_intel_firmware_update_config [MBIM_CID_INTEL_FIRMWARE_UPDATE_LAST] = {
+    { SET, NO_QUERY, NO_NOTIFY }, /* MBIM_CID_INTEL_FIRMWARE_UPDATE_MODEM_REBOOT */
+};
+
 /**
  * mbim_cid_can_set:
  * @service: a #MbimService.
@@ -189,6 +205,10 @@ mbim_cid_can_set (MbimService service,
         return cid_proxy_control_config[cid - 1].set;
     case MBIM_SERVICE_QMI:
         return cid_qmi_config[cid - 1].set;
+    case MBIM_SERVICE_ATDS:
+        return cid_atds_config[cid - 1].set;
+    case MBIM_SERVICE_INTEL_FIRMWARE_UPDATE:
+        return cid_intel_firmware_update_config[cid - 1].set;
     default:
         g_assert_not_reached ();
         return FALSE;
@@ -237,6 +257,10 @@ mbim_cid_can_query (MbimService service,
         return cid_proxy_control_config[cid - 1].query;
     case MBIM_SERVICE_QMI:
         return cid_qmi_config[cid - 1].query;
+    case MBIM_SERVICE_ATDS:
+        return cid_atds_config[cid - 1].query;
+    case MBIM_SERVICE_INTEL_FIRMWARE_UPDATE:
+        return cid_intel_firmware_update_config[cid - 1].query;
     default:
         g_assert_not_reached ();
         return FALSE;
@@ -285,6 +309,10 @@ mbim_cid_can_notify (MbimService service,
         return cid_proxy_control_config[cid - 1].notify;
     case MBIM_SERVICE_QMI:
         return cid_qmi_config[cid - 1].notify;
+    case MBIM_SERVICE_ATDS:
+        return cid_atds_config[cid - 1].notify;
+    case MBIM_SERVICE_INTEL_FIRMWARE_UPDATE:
+        return cid_intel_firmware_update_config[cid - 1].notify;
     default:
         g_assert_not_reached ();
         return FALSE;
@@ -308,10 +336,12 @@ mbim_cid_get_printable (MbimService service,
     /* CID = 0 is never a valid command */
     g_return_val_if_fail (cid > 0, NULL);
     /* Known service required */
-    g_return_val_if_fail (service > MBIM_SERVICE_INVALID, NULL);
+    g_return_val_if_fail (service >= MBIM_SERVICE_INVALID, NULL);
     g_return_val_if_fail (service < MBIM_SERVICE_LAST, NULL);
 
     switch (service) {
+    case MBIM_SERVICE_INVALID:
+        return "invalid";
     case MBIM_SERVICE_BASIC_CONNECT:
         return mbim_cid_basic_connect_get_string (cid);
     case MBIM_SERVICE_SMS:
@@ -334,8 +364,10 @@ mbim_cid_get_printable (MbimService service,
         return mbim_cid_proxy_control_get_string (cid);
     case MBIM_SERVICE_QMI:
         return mbim_cid_qmi_get_string (cid);
+    case MBIM_SERVICE_ATDS:
+        return mbim_cid_atds_get_string (cid);
     default:
         g_assert_not_reached ();
-        return FALSE;
+        return NULL;
     }
 }
